# ---------------------------------------------------------------------------
# - SEC1001.als                                                             -
# - afnix:sec module global test unit                                       -
# ---------------------------------------------------------------------------
# - This program is free software;  you can redistribute it  and/or  modify -
# - it provided that this copyright notice is kept intact.                  -
# -                                                                         -
# - This program  is  distributed in  the hope  that it will be useful, but -
# - without  any  warranty;  without  even   the   implied    warranty   of -
# - merchantability or fitness for a particular purpose.  In no event shall -
# - the copyright holder be liable for any  direct, indirect, incidental or -
# - special damages arising in any way out of the use of this software.     -
# ---------------------------------------------------------------------------
# - copyright (c) 1999-2008 amaury darsch                                   -
# ---------------------------------------------------------------------------

# @info   aes block mode test unit
# @author amaury darsch

# get the module
interp:library "afnix-sec"
interp:library "afnix-sio"

# this expression test an input cipher
const test-input-cipher (ice icd) {
  # verify input ciphers
  assert true (afnix:sec:input-cipher-p ice)
  assert true (afnix:sec:input-cipher-p icd)

  # the test string
  #const ts "afnix security module test suite"
  const ts "afnix"
  # create an input string
  const is (afnix:sio:InputString ts)
  # create an input/output stream
  const io (afnix:sio:InputOutput)
  # create an output string
  const os (afnix:sio:OutputString)
  
  # bind the input ciphers
  ice:set-is is
  icd:set-is io
  
  # encode the string
  while (ice:valid-p) (io:write (ice:read))
  # decode the string
  while (icd:valid-p) (os:write (icd:read))
  assert ts (os:to-string)
}

# create a symmetric key
const  key (afnix:sec:Key afnix:sec:Key:KSYM)
# create the encoding block cipher with a key
const  bce (afnix:sec:Aes key)
# create the decoding block cipher with a key
const  bcd (afnix:sec:Aes key true)

# create the input ciphers (ecb mode)
trans ice (afnix:sec:InputCipher bce)
trans icd (afnix:sec:InputCipher bcd)
# verify the input cipher parameters
assert afnix:sec:InputCipher:ECB (ice:get-mode)
assert afnix:sec:InputCipher:ECB (icd:get-mode)
# test the input cipher
test-input-cipher ice icd

# create the input ciphers (cbc mode)
trans ice (afnix:sec:InputCipher bce afnix:sec:InputCipher:CBC)
trans icd (afnix:sec:InputCipher bcd afnix:sec:InputCipher:CBC)
# verify the input cipher parameters
assert afnix:sec:InputCipher:CBC (ice:get-mode)
assert afnix:sec:InputCipher:CBC (icd:get-mode)
# bind the initial vector to the decoding cipher
icd:set-iv (ice:get-iv)
# test the input cipher
test-input-cipher ice icd

# create the input ciphers (cfb mode)
trans ice (afnix:sec:InputCipher bce afnix:sec:InputCipher:CFB)
trans icd (afnix:sec:InputCipher bcd afnix:sec:InputCipher:CFB)
# verify the input cipher parameters
assert afnix:sec:InputCipher:CFB (ice:get-mode)
assert afnix:sec:InputCipher:CFB (icd:get-mode)
# bind the initial vector to the decoding cipher
icd:set-iv (ice:get-iv)
# test the input cipher
test-input-cipher ice icd

# create the input ciphers (ofb mode)
trans ice (afnix:sec:InputCipher bce afnix:sec:InputCipher:OFB)
trans icd (afnix:sec:InputCipher bcd afnix:sec:InputCipher:OFB)
# verify the input cipher parameters
assert afnix:sec:InputCipher:OFB (ice:get-mode)
assert afnix:sec:InputCipher:OFB (icd:get-mode)
# bind the initial vector to the decoding cipher
icd:set-iv (ice:get-iv)
# test the input cipher
test-input-cipher ice icd
